@@ -5,6 +5,7 @@ from functools import wraps
 import inspect
 import json
 import operator
+import sys
 import warnings
 
 import numpy as np
@@ -14,7 +15,7 @@ from scipy.stats import t
 from . import Minimizer, Parameter, Parameters, lineshapes
 from .confidence import conf_interval
 from .jsonutils import HAS_DILL, decode4js, encode4js
-from .minimizer import validate_nan_policy, MinimizerResult
+from .minimizer import MinimizerResult
 from .printfuncs import ci_report, fit_report, fitreport_html_table
 
 # Use pandas.isnull for aligning missing data if pandas is available.
@@ -191,7 +192,7 @@ class Model(object):
     _hint_names = ('value', 'vary', 'min', 'max', 'expr')
 
     def __init__(self, func, independent_vars=None, param_names=None,
-                 nan_policy='raise', missing=None, prefix='', name=None, **kws):
+                 nan_policy='raise', prefix='', name=None, **kws):
         """Create a model from a user-supplied model function.
 
         The model function will normally take an independent variable
@@ -211,8 +212,6 @@ class Model(object):
         nan_policy : str, optional
             How to handle NaN and missing values in data. Must be one of
             'raise' (default), 'propagate', or 'omit'. See Note below.
-        missing : str, optional
-            Synonym for 'nan_policy' for backward compatibility.
         prefix : str, optional
             Prefix used for the model.
         name : str, optional
@@ -236,12 +235,7 @@ class Model(object):
 
            - 'propagate' : do nothing
 
-           -  'omit' : (was 'drop') drop missing data
-
-        4. The `missing` argument is deprecated in lmfit 0.9.8 and will be
-        removed in a later version. Use `nan_policy` instead, as it is
-        consistent with the Minimizer class.
-
+           -  'omit' : drop missing data
 
         Examples
         --------
@@ -271,12 +265,7 @@ class Model(object):
         self.independent_vars = independent_vars
         self._func_allargs = []
         self._func_haskeywords = False
-        if missing is not None:
-            nan_policy = missing
-            warnings.warn("The keyword 'missing' is deprecated as of lmfit "
-                          "0.9.13 and will be removed in the next release. "
-                          "Use 'nan_policy' instead.", FutureWarning)
-        self.nan_policy = validate_nan_policy(nan_policy)
+        self.nan_policy = nan_policy
 
         self.opts = kws
         self.param_hints = OrderedDict()
@@ -467,25 +456,43 @@ class Model(object):
         """Build parameters from function arguments."""
         if self.func is None:
             return
+        # need to fetch the following from the function signature:
+        #   pos_args: list of positional argument names
+        #   kw_args: dict of keyword arguments with default values
+        #   keywords_:  name of **kws argument or None
+        # 1. limited support for asteval functions as the model functions:
         if hasattr(self.func, 'argnames') and hasattr(self.func, 'kwargs'):
             pos_args = self.func.argnames[:]
+            keywords_ = None
             kw_args = {}
             for name, defval in self.func.kwargs:
                 kw_args[name] = defval
-            keywords_ = list(kw_args.keys())
+        # 2. modern, best-practice approach: use inspect.signature
+        elif sys.version_info > (3, 4):
+            pos_args = []
+            kw_args = {}
+            keywords_ = None
+            sig = inspect.signature(self.func)
+            for fnam, fpar in sig.parameters.items():
+                if fpar.kind == fpar.VAR_KEYWORD:
+                    keywords_ = fnam
+                elif fpar.kind == fpar.POSITIONAL_OR_KEYWORD:
+                    if fpar.default == fpar.empty:
+                        pos_args.append(fnam)
+                    else:
+                        kw_args[fnam] = fpar.default
+                elif fpar.kind == fpar.VAR_POSITIONAL:
+                    raise ValueError("varargs '*%s' is not supported" % fnam)
+        # 3. Py2 compatible approach
         else:
-            try:  # PY3
-                argspec = inspect.getfullargspec(self.func)
-                keywords_ = argspec.varkw
-            except AttributeError:  # PY2
-                argspec = inspect.getargspec(self.func)
-                keywords_ = argspec.keywords
-
+            argspec = inspect.getargspec(self.func)
+            keywords_ = argspec.keywords
             pos_args = argspec.args
             kw_args = {}
             if argspec.defaults is not None:
                 for val in reversed(argspec.defaults):
                     kw_args[pos_args.pop()] = val
+        # inspection done
 
         self._func_haskeywords = keywords_ is not None
         self._func_allargs = pos_args + list(kw_args.keys())
@@ -750,7 +757,15 @@ class Model(object):
         The "ravels" throughout are necessary to support pandas.Series.
 
         """
-        diff = self.eval(params, **kwargs) - data
+        model = self.eval(params, **kwargs)
+        if self.nan_policy == 'raise' and not np.all(np.isfinite(model)):
+            msg = ('The model function generated NaN values and the fit '
+                   'aborted! Please check your model function and/or set '
+                   'boundaries on parameters where applicable. In cases like '
+                   'this, using "nan_policy=\'omit\'" will probably not work.')
+            raise ValueError(msg)
+
+        diff = model - data
 
         if diff.dtype == np.complex:
             # data/model are complex
@@ -949,8 +964,8 @@ class Model(object):
         # All remaining kwargs should correspond to independent variables.
         for name in kwargs:
             if name not in self.independent_vars:
-                warnings.warn("The keyword argument %s does not" % name +
-                              "match any arguments of the model function." +
+                warnings.warn("The keyword argument %s does not " % name +
+                              "match any arguments of the model function. " +
                               "It will be ignored.", UserWarning)
 
         # If any parameter is not initialized raise a more helpful error.
@@ -979,7 +994,7 @@ class Model(object):
 
         # Handle null/missing values.
         if nan_policy is not None:
-            self.nan_policy = validate_nan_policy(nan_policy)
+            self.nan_policy = nan_policy
 
         mask = None
         if self.nan_policy == 'omit':
@@ -1346,7 +1361,7 @@ class ModelResult(Minimizer):
         if method is not None:
             self.method = method
         if nan_policy is not None:
-            self.nan_policy = validate_nan_policy(nan_policy)
+            self.nan_policy = nan_policy
 
         self.ci_out = None
         self.userargs = (self.data, self.weights)
@@ -1563,7 +1578,7 @@ class ModelResult(Minimizer):
                             show_correl=show_correl,
                             min_correl=min_correl, sort_pars=sort_pars)
         modname = self.model._reprstring(long=True)
-        return '[[Model]]\n    %s\n%s\n' % (modname, report)
+        return '[[Model]]\n    %s\n%s' % (modname, report)
 
     def _repr_html_(self, show_correl=True, min_correl=0.1):
         """Returns a HTML representation of parameters data."""
