@@ -4,6 +4,7 @@ from __future__ import division
 from collections import OrderedDict
 from copy import deepcopy
 import json
+import importlib
 
 from asteval import Interpreter, get_ast_names, valid_symbol_name
 from numpy import arcsin, array, cos, inf, isclose, nan, sin, sqrt
@@ -183,11 +184,41 @@ class Parameters(OrderedDict):
         # an Error. Another way of doing this would be to remove all the expr
         # from the Parameter instances before they get added, then to restore
         # them.
-        self._asteval.symtable.update(state['unique_symbols'])
+        # self._asteval.symtable.update(state['unique_symbols'])
 
+        symtab = self._asteval.symtable
+        for key, val in state['unique_symbols'].items():
+            if key not in symtab:
+                if isinstance(val, dict):
+                    value = val.get('__name__', None)
+                    symname = val.get('__name__', None)
+                    importer = val.get('importer', None)
+                    if value is None and symname is not None and importer is not None:
+                        _mod = importlib.import_module(importer)
+                        value = getattr(_mod, symname, None)
+                    if value is not None:
+                        symtab[key] = value
+                else:
+                    symtab[key] = val
         # then add all the parameters
         self.add_many(*state['params'])
 
+    def eval(self, expr):
+        """Evaluate a statement using the asteval Interpreter.
+
+        Parameters
+        ----------
+        expr : string
+            An expression containing parameter names and other symbols
+            recognizable by the asteval Interpreter.
+
+        Returns
+        -------
+           The result of the expression.
+
+        """
+        return self._asteval.eval(expr)
+
     def update_constraints(self):
         """Update all constrained parameters, checking that dependencies are
         evaluated as needed."""
@@ -356,12 +387,16 @@ class Parameters(OrderedDict):
         >>> params.add_many(f, g)
 
         """
-        for para in parlist:
-            if isinstance(para, Parameter):
-                self.__setitem__(para.name, para)
-            else:
-                param = Parameter(*para)
-                self.__setitem__(param.name, param)
+        __params = []
+        for par in parlist:
+            if not isinstance(par, Parameter):
+                par = Parameter(*par)
+            __params.append(par)
+            par._delay_asteval = True
+            self.__setitem__(par.name, par)
+
+        for para in __params:
+            para._delay_asteval = False
 
     def valuesdict(self):
         """Return an ordered dictionary of parameter values.
@@ -373,7 +408,7 @@ class Parameters(OrderedDict):
            Parameter.
 
         """
-        return OrderedDict(((p.name, p.value) for p in self.values()))
+        return OrderedDict((p.name, p.value) for p in self.values())
 
     def dumps(self, **kws):
         """Represent Parameters as a JSON string.
