@@ -7,7 +7,8 @@ from numpy.testing import assert_allclose, assert_almost_equal
 import pytest
 
 from lmfit import Parameters, minimize
-from lmfit.models import VoigtModel
+from lmfit.lineshapes import exponential
+from lmfit.models import ExponentialModel, VoigtModel
 
 
 def check(para, real_val, sig=3):
@@ -108,52 +109,55 @@ def test_bounds_expression():
                         -4.390334984618851e-05, decimal=6)
 
 
-def test_numdifftools_no_bounds():
+@pytest.mark.parametrize("fit_method", ['nelder', 'lbfgs'])
+def test_numdifftools_no_bounds(fit_method):
     pytest.importorskip("numdifftools")
-    # load data to be fitted
-    data = np.loadtxt(os.path.join(os.path.dirname(__file__), '..', 'examples',
-                                   'test_peak.dat'))
-    x = data[:, 0]
-    y = data[:, 1]
 
-    # define the model and initialize parameters
-    mod = VoigtModel()
+    np.random.seed(7)
+    x = np.linspace(0, 100, num=50)
+    noise = np.random.normal(scale=0.25, size=x.size)
+    y = exponential(x, amplitude=5, decay=15) + noise
+
+    mod = ExponentialModel()
     params = mod.guess(y, x=x)
-    params['sigma'].set(min=-np.inf)
 
     # do fit, here with leastsq model
     result = mod.fit(y, params, x=x, method='leastsq')
 
-    for fit_method in ['nelder', 'basinhopping', 'ampgo']:
-        result_ndt = mod.fit(y, params, x=x, method=fit_method)
+    result_ndt = mod.fit(y, params, x=x, method=fit_method)
 
-        # assert that fit converged to the same result
-        vals = [result.params[p].value for p in result.params.valuesdict()]
-        vals_ndt = [result_ndt.params[p].value for p in result_ndt.params.valuesdict()]
-        assert_allclose(vals_ndt, vals, rtol=0.1)
-        assert_allclose(result_ndt.chisqr, result.chisqr, rtol=1e-5)
+    # assert that fit converged to the same result
+    vals = [result.params[p].value for p in result.params.valuesdict()]
+    vals_ndt = [result_ndt.params[p].value for p in result_ndt.params.valuesdict()]
+    assert_allclose(vals_ndt, vals, rtol=0.1)
+    assert_allclose(result_ndt.chisqr, result.chisqr, rtol=1e-5)
 
-        # assert that parameter uncertaintes from leastsq and calculated from
-        # the covariance matrix using numdifftools are very similar
-        stderr = [result.params[p].stderr for p in result.params.valuesdict()]
-        stderr_ndt = [result_ndt.params[p].stderr for p in result_ndt.params.valuesdict()]
+    # assert that parameter uncertaintes from leastsq and calculated from
+    # the covariance matrix using numdifftools are very similar
+    stderr = [result.params[p].stderr for p in result.params.valuesdict()]
+    stderr_ndt = [result_ndt.params[p].stderr for p in result_ndt.params.valuesdict()]
 
-        perr = np.array(stderr) / np.array(vals)
-        perr_ndt = np.array(stderr_ndt) / np.array(vals_ndt)
-        assert_almost_equal(perr_ndt, perr, decimal=4)
+    perr = np.array(stderr) / np.array(vals)
+    perr_ndt = np.array(stderr_ndt) / np.array(vals_ndt)
+    assert_almost_equal(perr_ndt, perr, decimal=3)
 
-        # assert that parameter correlatations from leastsq and calculated from
-        # the covariance matrix using numdifftools are very similar
-        for par1 in result.var_names:
-            cor = [result.params[par1].correl[par2] for par2 in
-                   result.params[par1].correl.keys()]
-            cor_ndt = [result_ndt.params[par1].correl[par2] for par2 in
-                       result_ndt.params[par1].correl.keys()]
-            assert_almost_equal(cor_ndt, cor, decimal=2)
+    # assert that parameter correlatations from leastsq and calculated from
+    # the covariance matrix using numdifftools are very similar
+    for par1 in result.var_names:
+        cor = [result.params[par1].correl[par2] for par2 in
+               result.params[par1].correl.keys()]
+        cor_ndt = [result_ndt.params[par1].correl[par2] for par2 in
+                   result_ndt.params[par1].correl.keys()]
+        assert_almost_equal(cor_ndt, cor, decimal=2)
 
 
-def test_numdifftools_with_bounds():
+@pytest.mark.parametrize("fit_method", ['nelder', 'basinhopping', 'ampgo',
+                                        'shgo', 'dual_annealing'])
+def test_numdifftools_with_bounds(fit_method):
     pytest.importorskip("numdifftools")
+    if fit_method in ['shgo', 'dual_annealing']:
+        pytest.importorskip("scipy", minversion="1.2")
+
     # load data to be fitted
     data = np.loadtxt(os.path.join(os.path.dirname(__file__), '..', 'examples',
                                    'test_peak.dat'))
@@ -164,38 +168,37 @@ def test_numdifftools_with_bounds():
     mod = VoigtModel()
     params = mod.guess(y, x=x)
     params['amplitude'].set(min=25, max=70)
-    params['sigma'].set(min=0, max=1)
+    params['sigma'].set(max=1)
     params['center'].set(min=5, max=15)
 
     # do fit, here with leastsq model
     result = mod.fit(y, params, x=x, method='leastsq')
 
-    for fit_method in ['nelder', 'basinhopping', 'ampgo']:
-        result_ndt = mod.fit(y, params, x=x, method=fit_method)
+    result_ndt = mod.fit(y, params, x=x, method=fit_method)
 
-        # assert that fit converged to the same result
-        vals = [result.params[p].value for p in result.params.valuesdict()]
-        vals_ndt = [result_ndt.params[p].value for p in result_ndt.params.valuesdict()]
-        assert_allclose(vals_ndt, vals, rtol=0.1)
-        assert_allclose(result_ndt.chisqr, result.chisqr, rtol=1e-5)
+    # assert that fit converged to the same result
+    vals = [result.params[p].value for p in result.params.valuesdict()]
+    vals_ndt = [result_ndt.params[p].value for p in result_ndt.params.valuesdict()]
+    assert_allclose(vals_ndt, vals, rtol=0.1)
+    assert_allclose(result_ndt.chisqr, result.chisqr, rtol=1e-5)
 
-        # assert that parameter uncertaintes from leastsq and calculated from
-        # the covariance matrix using numdifftools are very similar
-        stderr = [result.params[p].stderr for p in result.params.valuesdict()]
-        stderr_ndt = [result_ndt.params[p].stderr for p in result_ndt.params.valuesdict()]
+    # assert that parameter uncertaintes from leastsq and calculated from
+    # the covariance matrix using numdifftools are very similar
+    stderr = [result.params[p].stderr for p in result.params.valuesdict()]
+    stderr_ndt = [result_ndt.params[p].stderr for p in result_ndt.params.valuesdict()]
 
-        perr = np.array(stderr) / np.array(vals)
-        perr_ndt = np.array(stderr_ndt) / np.array(vals_ndt)
-        assert_almost_equal(perr_ndt, perr, decimal=4)
+    perr = np.array(stderr) / np.array(vals)
+    perr_ndt = np.array(stderr_ndt) / np.array(vals_ndt)
+    assert_almost_equal(perr_ndt, perr, decimal=3)
 
-        # assert that parameter correlatations from leastsq and calculated from
-        # the covariance matrix using numdifftools are very similar
-        for par1 in result.var_names:
-            cor = [result.params[par1].correl[par2] for par2 in
-                   result.params[par1].correl.keys()]
-            cor_ndt = [result_ndt.params[par1].correl[par2] for par2 in
-                       result_ndt.params[par1].correl.keys()]
-            assert_almost_equal(cor_ndt, cor, decimal=2)
+    # assert that parameter correlatations from leastsq and calculated from
+    # the covariance matrix using numdifftools are very similar
+    for par1 in result.var_names:
+        cor = [result.params[par1].correl[par2] for par2 in
+               result.params[par1].correl.keys()]
+        cor_ndt = [result_ndt.params[par1].correl[par2] for par2 in
+                   result_ndt.params[par1].correl.keys()]
+        assert_almost_equal(cor_ndt, cor, decimal=2)
 
 
 def test_numdifftools_calc_covar_false():
